@@ -239,7 +239,6 @@ __NR_epoll_ctl = Constant("__NR_epoll_ctl", 233)
 __NR_tgkill = Constant("__NR_tgkill", 234)
 __NR_utimes = Constant("__NR_utimes", 235)
 __NR_vserver = Constant("__NR_vserver", 236)
-__NR_vserver = Constant("__NR_vserver", 236)
 __NR_mbind = Constant("__NR_mbind", 237)
 __NR_set_mempolicy = Constant("__NR_set_mempolicy", 238)
 __NR_get_mempolicy = Constant("__NR_get_mempolicy", 239)
@@ -306,6 +305,37 @@ __NR_recvmmsg = Constant("__NR_recvmmsg", 299)
 __NR_fanotify_init = Constant("__NR_fanotify_init", 300)
 __NR_fanotify_mark = Constant("__NR_fanotify_mark", 301)
 __NR_prlimit64 = Constant("__NR_prlimit64", 302)
+__NR_name_to_handle_at = Constant("__NR_name_to_handle_at", 303)
+__NR_open_by_handle_at = Constant("__NR_open_by_handle_at", 304)
+__NR_clock_adjtime = Constant("__NR_clock_adjtime", 305)
+__NR_syncfs = Constant("__NR_syncfs", 306)
+__NR_sendmmsg = Constant("__NR_sendmmsg", 307)
+__NR_setns = Constant("__NR_setns", 308)
+__NR_getcpu = Constant("__NR_getcpu", 309)
+__NR_process_vm_readv = Constant("__NR_process_vm_readv", 310)
+__NR_process_vm_writev = Constant("__NR_process_vm_writev", 311)
+__NR_kcmp = Constant("__NR_kcmp", 312)
+__NR_finit_module = Constant("__NR_finit_module", 313)
+__NR_sched_setattr = Constant("__NR_sched_setattr", 314)
+__NR_sched_getattr = Constant("__NR_sched_getattr", 315)
+__NR_renameat2 = Constant("__NR_renameat2", 316)
+__NR_seccomp = Constant("__NR_seccomp", 317)
+__NR_getrandom = Constant("__NR_getrandom", 318)
+__NR_memfd_create = Constant("__NR_memfd_create", 319)
+__NR_kexec_file_load = Constant("__NR_kexec_file_load", 320)
+__NR_bpf = Constant("__NR_bpf", 321)
+__NR_execveat = Constant("__NR_execveat", 322)
+__NR_userfaultfd = Constant("__NR_userfaultfd", 323)
+__NR_membarrier = Constant("__NR_membarrier", 324)
+__NR_mlock2 = Constant("__NR_mlock2", 325)
+__NR_copy_file_range = Constant("__NR_copy_file_range", 326)
+__NR_preadv2 = Constant("__NR_preadv2", 327)
+__NR_pwritev2 = Constant("__NR_pwritev2", 328)
+__NR_pkey_mprotect = Constant("__NR_pkey_mprotect", 329)
+__NR_pkey_alloc = Constant("__NR_pkey_alloc", 330)
+__NR_pkey_free = Constant("__NR_pkey_free", 331)
+__NR_statx = Constant("__NR_statx", 332)
+
 __SYS_NERR = Constant("__SYS_NERR", ((129) + 1))
 _SYS_TIME_H = Constant("_SYS_TIME_H", 1)
 SYS_accept = Constant("SYS_accept", 43)
